
--[[
BLIB
Button Library
Created for creating buttons/textinput with ease
This library was created by Bruce Berrios
]]--

button = {}
imgbutton = {}
textbox = {}
progress = {}
scrollframe = {}

textboxtimer = 0
delay = 0.15

function createNewButton(x,y,width,height,id,text,text2,font,font2,state,sound,msound,r,g,b,mr,mg,mb)
  r = r/255 or 1
  g = g/255 or 1
  b = b/255 or 1
  table.insert(button,{x=x,y=y,width=width,height=height,id=id,text=text,text2=text2,font=font,font2=font2,mouseover=false,state=state,sound=sound,msound=msound,r=r,g=g,b=b,mr=mr,mg=mg,mb=mb})
end
function createNewImageButton(img,x,y,id,text,text2,font,font2,state,sound,msound,r,g,b,mr,mg,mb,rot)
  r = r/255 or 1
  g = g/255 or 1
  b = b/255 or 1
  table.insert(imgbutton,{img=img,x=x,y=y,id=id,text=text,text2=text2,font=font,font2=font2,mouseover=false,state=state,sound=sound,msound=msound,r=r,g=g,b=b,mr=mr,mg=mg,mb=mb,rot=rot})
end

function createNewTextbox(x,y,width,height,id,text,font,state,r,g,b,max,min)
  r = r/255 or 1
  g = g/255 or 1
  b = b/255 or 1
  table.insert(textbox,{x=x,y=y,width=width,height=height,id=id,text=text,font=font,state=state,r=r,g=g,b=b,focus=false,max=max,min=min})
end

function createNewProgressBar(x,y,width,height,id,text,font,state,r,g,b,prog,max,min)
  r = r/255 or 1
  g = g/255 or 1
  b = b/255 or 1
  table.insert(progress,{x=x,y=y,width=width,height=height,id=id,text=text,font=font,state=state,r=r,g=g,b=b,prog=prog,max=max,min=min})
end

function createNewScrollFrame(x,y,width,height,id,state,scale)
  table.insert(scrollframe,{x=x,y=y,width=width,height=height,x1=x1,y1=y1,w1=w1,h1=h1,id=id,state=state,mousedown = false,clip = 0,scale = scale})
end

function drawScrollFrame()
  for _,v in ipairs(scrollframe) do
    if v.state == gamestate then
      love.graphics.setColor(84/255,84/255,84/255,255)
      love.graphics.rectangle("fill",v.x,v.y,v.width,v.height)
      love.graphics.setColor(191/255,191/255,191/255,255)
      v.x1 = v.x
      if not v.mousedown then
        v.y1 = v.y1
      else
        v.y1 = v.mousey
      end
      v.w1 = v.width
      v.h1 = v.height/v.scale
      love.graphics.rectangle("fill",v.x1,v.clip,v.w1,v.h1)
    end
  end
end

function clipScroll()
  for _,v in ipairs(scrollframe) do
    if v.state == gamestate then
      love.graphics.setColor(255,255,255,105/255)
      love.graphics.setScissor(screenwidth/2.9,v.y,screenwidth/2,v.height)
      love.graphics.rectangle("fill",screenwidth/2.9,v.y,screenwidth/2,v.height)
      posHats()
      love.graphics.setColor(255,255,255,255)
      love.graphics.setScissor()
    end
  end
end

function updateScroll()
  for _,v in ipairs(scrollframe) do
    if v.state == gamestate then
      currentclip = v.clip
      v.h1 = v.height/v.scale
      if v.clip >= v.y + v.height - v.h1 then
        v.clip = v.y + v.height - v.h1
      end
      if v.clip < v.y then
        v.clip = v.y
      end
    end
  end
end

function mouseScroll(x,y,button)
  for _,v in ipairs(scrollframe) do
    if v.state == gamestate then
      if button == "wu" then
        v.clip = v.clip - 5
        --v.mousedown = true
      end
      if button == "wd" then
        v.clip = v.clip + 5
        --v.mousedown = true
      end
      if button == "n" then
        v.clip = v.clip
        v.mousedown = false
      end
    end
  end
end

function clickScroll()
  for _,v in ipairs(scrollframe) do
    x = love.mouse.getX()
    y = love.mouse.getY()
    v.x1 = v.x
    v.w1 = v.width
    v.h1 = v.height/v.scale
    if love.mouse.isDown(1) and x > v.x1 and
      x < v.x + v.width and
      y > v.y and
      y < v.y + v.height then
      v.mousedown = true
      v.clip = mousey
    else
      v.mousedown = false
    end
  end
end

function drawProgress()
  for _,v in ipairs(progress) do
    if v.state == gamestate then
      love.graphics.setColor(v.r,v.g,v.b,255)
      if v.prog >= v.max then
        love.graphics.rectangle("fill",v.x,v.y,v.width,v.height)
      elseif v.prog < v.max and v.prog > v.min then
        love.graphics.rectangle("fill",v.x,v.y,v.width/v.max*v.prog,v.height)
      elseif v.prog <= v.min then
        love.graphics.rectangle("fill",v.x,v.y,v.min,v.height)
      end
      love.graphics.setColor(0,0,0,255)
      love.graphics.rectangle("line",v.x,v.y,v.width,v.height)
      love.graphics.setFont(v.font)
      love.graphics.print(v.text,v.x,v.y)
      love.graphics.setColor(255,255,255,255)
    end
  end
end

function updateProgress()
  for _,v in ipairs(progress) do
    if v.id == "health" then
      v.prog = player.health
    elseif v.id == "tacos" then
      v.prog = tc
      v.max = goal
    elseif v.id == "bhealth" then
      v.prog = boss.health
      if not bossDetermined then
        v.x = 9999
      elseif bossDetermined then
        v.x = screenwidth/1.2-70/2
      end
    end
  end
end

function drawTextbox()
  for _,v in ipairs(textbox) do
    if v.state == gamestate then
      love.graphics.setColor(v.r,v.g,v.b,255)
      love.graphics.rectangle("fill",v.x,v.y,v.width,v.height)
      love.graphics.setColor(0,0,0,255)
      love.graphics.setFont(v.font)
      love.graphics.print(v.text,v.x+2,v.y+5)
      love.graphics.setColor(255,255,255,255)
    end
  end
end

function textboxFocus(x,y)
  for _,v in ipairs(textbox)do
    if v.state == gamestate then
      if x > v.x and
        x < v.x + v.width and
        y > v.y and
        y < v.y + v.height then
        if v.id == "key" then
          v.text = ""
        end
        if v.focus == false then
          v.focus = true
          userKey = v.text
        elseif v.focus == true then
          v.focus = false
          userKey = v.text
        end
      else
        v.focus = false
        userKey = v.text
      end
    end
  end
end

function textboxType(dt)
  textboxtimer = textboxtimer + dt
  for _,v in ipairs(textbox) do
    if v.state == gamestate then
      if v.focus then
        if textboxtimer >= delay and love.keyboard.isDown("a") and string.len(v.text) < v.max then
          v.text = v.text .. "a"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("b") and string.len(v.text) < v.max then
          v.text = v.text .. "b"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("c") and string.len(v.text) < v.max then
          v.text = v.text .. "c"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("d") and string.len(v.text) < v.max then
          v.text = v.text .. "d"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("e") and string.len(v.text) < v.max then
          v.text = v.text .. "e"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("f") and string.len(v.text) < v.max then
          v.text = v.text .. "f"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("g") and string.len(v.text) < v.max then
          v.text = v.text .. "g"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("h") and string.len(v.text) < v.max then
          v.text = v.text .. "h"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("i") and string.len(v.text) < v.max then
          v.text = v.text .. "i"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("j") and string.len(v.text) < v.max then
          v.text = v.text .. "j"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("k") and string.len(v.text) < v.max then
          v.text = v.text .. "k"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("l") and string.len(v.text) < v.max then
          v.text = v.text .. "l"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("m") and string.len(v.text) < v.max then
          v.text = v.text .. "m"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("n") and string.len(v.text) < v.max then
          v.text = v.text .. "n"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("o") and string.len(v.text) < v.max then
          v.text = v.text .. "o"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("p") and string.len(v.text) < v.max then
          v.text = v.text .. "p"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("q") and string.len(v.text) < v.max then
          v.text = v.text .. "q"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("r") and string.len(v.text) < v.max then
          v.text = v.text .. "r"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("s") and string.len(v.text) < v.max then
          v.text = v.text .. "s"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("t") and string.len(v.text) < v.max then
          v.text = v.text .. "t"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("u") and string.len(v.text) < v.max then
          v.text = v.text .. "u"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("v") and string.len(v.text) < v.max then
          v.text = v.text .. "v"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("w") and string.len(v.text) < v.max then
          v.text = v.text .. "w"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("x") and string.len(v.text) < v.max then
          v.text = v.text .. "x"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("y") and string.len(v.text) < v.max then
          v.text = v.text .. "y"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("z") and string.len(v.text) < v.max then
          v.text = v.text .. "z"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("0") and string.len(v.text) < v.max or textboxtimer >= delay and love.keyboard.isDown("kp0") and string.len(v.text) < v.max then
          v.text = v.text .. "0"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("1") and string.len(v.text) < v.max or textboxtimer >= delay and love.keyboard.isDown("kp1") and string.len(v.text) < v.max then
          v.text = v.text .. "1"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("2") and string.len(v.text) < v.max or textboxtimer >= delay and love.keyboard.isDown("kp2") and string.len(v.text) < v.max then
          v.text = v.text .. "2"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("3") and string.len(v.text) < v.max or textboxtimer >= delay and love.keyboard.isDown("kp3") and string.len(v.text) < v.max then
          v.text = v.text .. "3"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("4") and string.len(v.text) < v.max or textboxtimer >= delay and love.keyboard.isDown("kp4") and string.len(v.text) < v.max then
          v.text = v.text .. "4"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("5") and string.len(v.text) < v.max or textboxtimer >= delay and love.keyboard.isDown("kp5") and string.len(v.text) < v.max then
          v.text = v.text .. "5"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("6") and string.len(v.text) < v.max or textboxtimer >= delay and love.keyboard.isDown("kp6") and string.len(v.text) < v.max then
          v.text = v.text .. "6"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("7") and string.len(v.text) < v.max or textboxtimer >= delay and love.keyboard.isDown("kp7") and string.len(v.text) < v.max then
          v.text = v.text .. "7"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("8") and string.len(v.text) < v.max or textboxtimer >= delay and love.keyboard.isDown("kp8") and string.len(v.text) < v.max then
          v.text = v.text .. "8"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("9") and string.len(v.text) < v.max or textboxtimer >= delay and love.keyboard.isDown("kp9") and string.len(v.text) < v.max then
          v.text = v.text .. "9"
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("space") and string.len(v.text) < v.max then
          v.text = v.text .. " "
          textboxtimer = 0
        elseif textboxtimer >= delay and love.keyboard.isDown("backspace") and string.len(v.text) > v.min then
          textboxtimer = 0
          v.text = v.text:sub(1,#v.text-1)
        end
      end
    end
  end
end

function drawButtons()
  for _,v in ipairs(button) do
    if v.state == gamestate then
      if v.id == "yes" then
        v.x = lx4 + v.width * 1.2
        v.y = ly4 + v.height * 1.5
      end
      if v.id == "bhat" then
          v.x = bx
          v.y = by
          v.width = bxw
          v.height = byh
          if selected == 0 then
            v.text = "Select a Hat"
          elseif selected == "hat1" then
            v.text = "Almost Earth"
          elseif selected == "hat2" then
            v.text = "Lovely Antennas"
          elseif selected == "hat3" then
            v.text = "Blue Balloon"
          elseif selected == "hat4" then
            v.text = "Dark Crown"
          elseif selected == "hat5" then
            v.text = "Rubber Ducky"
          elseif selected == "hat6" then
            v.text = "Head Stack"
          elseif selected == "hat7" then
            v.text = "Light Bulb"
          elseif selected == "hat8" then
            v.text = "Rainbow Crown"
          elseif selected == "hat9" then
            v.text = "Top Hat"
          elseif selected == "hat10" then
            v.text = "Troll"
          end
      end
      if not v.mouseover then
        love.graphics.setColor(v.r,v.g,v.b,255)
        love.graphics.rectangle("fill",v.x,v.y,v.width,v.height)
        love.graphics.setColor(0,0,0,255)
        love.graphics.setFont(v.font)
        love.graphics.print(v.text,v.x+20,v.y+15)
        love.graphics.setColor(255,255,255,255)
      elseif v.mouseover then
        love.graphics.setColor(v.mr,v.mg,v.mb,255)
        love.graphics.rectangle("fill",v.x,v.y,v.width,v.height)
        love.graphics.setColor(255,255,255,255)
         if v.text2 ~= nil then
            love.graphics.setFont(v.font2)
            love.graphics.print(v.text2,v.x+25,v.y+10)
            love.graphics.setColor(255,255,255,255)
          else
            love.graphics.setFont(v.font2)
            love.graphics.print(v.text,v.x+25,v.y+10)
            love.graphics.setColor(255,255,255,255)
          end
      end
    end
  end
  for _,v in ipairs(imgbutton) do
    if v.state == gamestate then
      if v.id == "mute" then
        if not muted then
          love.graphics.draw(v.img,mplay,v.x,v.y)
          music:setVolume(1)
          if v.mouseover then
            love.graphics.setColor(v.mr,v.mg,v.mb,255)
            v.img:setFilter("nearest","nearest")
            love.graphics.draw(v.img,mplay,v.x+v.img:getWidth()/2,v.y+v.img:getHeight()/2,0,1,1,v.img:getWidth()/2,v.img:getHeight()/2)
            love.graphics.setColor(255,255,255,255)
          end
        elseif muted then
          love.graphics.draw(v.img,mmute,v.x,v.y)
          music:setVolume(0)
          if v.mouseover then
            love.graphics.setColor(v.mr,v.mg,v.mb,255)
            v.img:setFilter("nearest","nearest")
            love.graphics.draw(v.img,mmute,v.x+v.img:getWidth()/2,v.y+v.img:getHeight()/2,0,1,1,v.img:getWidth()/2,v.img:getHeight()/2)
            love.graphics.setColor(255,255,255,255)
          end
        end
      end
      if not v.mouseover and v.id ~= "mute" then
        love.graphics.setColor(v.r,v.g,v.b,255)
        v.img:setFilter("nearest","nearest")
        love.graphics.draw(v.img,v.x,v.y)
        love.graphics.setColor(0,0,0,1)
        love.graphics.setFont(v.font)
        love.graphics.print(v.text,v.x+20,v.y+10,v.rot)
        love.graphics.setColor(1, 1, 1, 1)
      elseif v.mouseover and v.id ~= "mute" then
        love.graphics.setColor(v.mr/255,v.mg/255,v.mb/255,255)
        v.img:setFilter("nearest","nearest")
        love.graphics.draw(v.img,v.x+v.img:getWidth()/2,v.y+v.img:getHeight()/2,0,1,1,v.img:getWidth()/2,v.img:getHeight()/2)
        love.graphics.setColor(1,1, 1, 1)
         if v.text2 ~= nil then
            love.graphics.setFont(v.font2)
            love.graphics.print(v.text2,v.x+25,v.y+10,v.rot)
            love.graphics.setColor(1,1,1,1)
          else
            love.graphics.setFont(v.font2)
            love.graphics.print(v.text,v.x+25,v.y+10,v.rot)
            love.graphics.setColor(1, 1, 1, 1)
          end
      end
    end
  end
end

function buttonClicked(x,y)
  for _,v in ipairs(button) do
    if v.state == gamestate then
      if x > v.x and
        x < v.x + v.width and
        y > v.y and
        y < v.y + v.height then
        if v.sound ~= nil then
          v.sound:play()
        end
        if v.id == "yes" then
          music:setPitch(1)
          if not muted then
            music:play()
          end
          boss.intro:stop()
          boss.die:stop()
          boss.win:stop()
          yay:stop()
          boo:stop()
          boss.music:stop()
          removeInstances()
          player.dead = false
          tc = 0
          bombTimer = 0
          lTimer = 0
          gTimer = 0
          dTimer = 0
          quitclicked = false
          paused = false
          currentboss = ""
          bossDetermined = false
          boss.dead = false
          bossbattle = false
          fc = 1
          tspeed = 0
          gspeed = 0.1
          booplayed = false
          yayplayed = false
          dunplayed = false
          boss.diesaid = false
          boss.introsaid = false
          boss.winsaid = false
          player.health = 100
          player.x = screenwidth/2-player.width/2
          player.y = screenheight/2-player.height/2
          boss.x = screenwidth/2 - boss.width/2
          boss.y = screenheight/2 - boss.height/2
          lyw = screenwidth/2
          lyh = screenheight/4
          lx4 = screenwidth/2 - lyw/2
          ly4 = 0 - lyh
          gamestate = "loading"
        end
      end
    end
  end
  for _,v in ipairs(imgbutton) do
    if v.state == gamestate then
      if x > v.x and
        x < v.x + v.img:getWidth() and
        y > v.y and
        y < v.y + v.img:getHeight() then
        if v.sound ~= nil then
          v.sound:stop()
          v.sound:play()
        end
        if v.id  == "play" then
          gamestate = "select"
          current1 = grass
          current2 = mount
          current3 = mountquad
        elseif v.id == "actkey" then
          checkKey()
        elseif v.id == "load" then
          gamestate = "game"
        elseif v.id == "inf" then
          gamestate = "game2"
        elseif v.id == "shop" then
          music:stop()
          thrift:play()
          gamestate = "shop"
          player.mhat = player.hatv
        elseif v.id == "opt" then
          gamestate = "options"
        elseif v.id == "mute" then
          if muted then
            muted = false
          elseif not muted then
            muted = true
          end
        elseif v.id == "quit" then
          love.quit()
        elseif v.id == "back1" or v.id == "back2" or v.id == "back3" then
          thrift:stop()
          music:play()
          selected = 0
          player.mhat = 0
          ability = "Special Ability"
          gamestate = "menu"
        elseif v.id == "back4" then
          music:setPitch(1)
          if not muted then
            music:play()
          end
          bombTimer = 0
          boss.intro:stop()
          boss.die:stop()
          boss.win:stop()
          yay:stop()
          removeInstances()
          boo:stop()
          boss.music:stop()
          player.dead = false
          tc = 0
          lTimer = 0
          gTimer = 0
          dTimer = 0
          quitclicked = false
          paused = false
          currentboss = ""
          bossDetermined = false
          bossbattle = false
          fc = 1
          tspeed = 0
          gspeed = 0.1
          booplayed = false
          yayplayed = false
          dunplayed = false
          boss.diesaid = false
          boss.dead = false
          boss.introsaid = false
          boss.winsaid = false
          player.health = 100
          player.x = screenwidth/2-player.width/2
          player.y = screenheight/2-player.height/2
          boss.x = screenwidth/2 - boss.width/2
          boss.y = screenheight/2 - boss.height/2
          lyw = screenwidth/2
          lyh = screenheight/4
          lx4 = screenwidth/2 - lyw/2
          ly4 = 0 - lyh
          gamestate = "loading"
        elseif v.id == "leave" then
          print(quitclicked)
          if not quitclicked and not player.dead then
            quitclicked = true
            elseif quitclicked and not player.dead then
            quitclicked = false
          end
        elseif v.id == "ileave" then
          print(paused)
          if not paused and not player.dead then
            paused = true
            elseif paused and not player.dead then
            paused = false
          end
        end
      end
    end
  end
end

function buttonMouse()
  for _,v in ipairs(button) do
    if v.state == gamestate then
      if mousex > v.x and
        mousex < v.x + v.width and
        mousey > v.y and
        mousey < v.y + v.height then
        if v.mouseover == false then
          if v.msound ~= nil then
            v.msound:stop()
            v.msound:play()
          else
          end
        end
        v.mouseover = true
      else
        v.mouseover = false
      end
    end
  end
  for _,v in ipairs(imgbutton) do
    if v.state == gamestate then
      if mousex > v.x and
        mousex < v.x + v.img:getWidth() and
        mousey > v.y and
        mousey < v.y + v.img:getHeight() then
        if v.mouseover == false then
          if v.msound ~= nil then
            v.msound:stop()
            v.msound:play()
          else
          end
          if v.id == "level1" then
            current1 = grass
            current2 = mount
            current3 = mountquad
            goal = 2000
            currentboss = "narwhal"
          end
          if v.id == "level2" then
            current1 = street
            current2 = buildin
            current3 = buildquad
            goal = 4000
            currentboss = "donut"
          end
        end
        v.mouseover = true
      else
        v.mouseover = false
      end
    end
  end
end
