

--[[

GSLIB

Gif Loading/Playing Library

Created for loading/playing gifs with ease

This library was created by Bruce Berrios

------------------------------------------

Basic Tutorial:

------------------------------------------

  Minimal Template:



  GS = require"GSLIB"



  function love.load()

    MyGifDir = "Folder1/GIFS/MyGif's Frames/" -- The Directory to the folder that contains all your gif's frames

    GS:Init("GSLIB")

    MyGif = GS.NewGif(MyGifDir,loop) -- loop can be either true(Play Gif Infinitely Many Times) or false(Play Gif Once)

  end



  function love.update(dt)

    MyGif.UpdateGif(dt,speed) -- dt is Delta Time, speed is the speed of the gif(Number Higher = Slower, Number Lower = Faster, Its recommended to use DeltaTime)

  end



  function love.draw()

    MyGif.Draw(x,y,rt,sx,sy) -- x is The X Coordinate of the gif, y is the Y Coordinate of the gif, rt is the rotation of the gif, sx is the ScaleX of the gif, sy is the ScaleY of the gif

  end



  -- Thats it!

]]--





print("GSLIB LOADED...")



local gs = {}

gs.__index = gs



local i = ""



function gs:Init(x)

  i = x

  print(x.." Initialized!")

end



setmetatable(gs, {

    _call = function(cls,...)

      return cls.new(...)

    end,

})



function gs.NewGif(init,loop)

  local self = setmetatable({},gs)

  local frames = love.filesystem.getDirectoryItems(init)

  print("Loading Frames From..."..init)

  self.image = {}

  self.index = 1

  self.timer = 0

  self.loop = loop


  local pos = 1
  for i = 1, #frames do
    if(string.sub(frames[i], #frames[i]-3, #frames[i]) == ".png") then
      self.image[pos] = love.graphics.newImage(init..frames[i])
      pos = pos + 1

    end
  end

  self.frames = #self.image

  return self

end



function gs.UpdateGif(self,dt,speed)

  self.timer = self.timer + dt

  if self.timer >= speed then

    if self.loop then

      if self.index >= self.frames then

        self.index = 1

      elseif self.index <= self.frames then

        self.index = self.index + 1

      end

    elseif not self.loop then

      if self.index >= self.frames then

        self.index = self.frames

      elseif self.index <= self.frames then

        self.index = self.index + 1

      end

    end

    self.timer = 0

  end

end



function gs.DrawGif(self,x,y,rt,sx,sy)
  love.graphics.draw(self.image[self.index],x,y,rt,sx,sy)
end



return gs
